/*
 * Smithsonian Astrophysical Observatory, Cambridge, MA, USA
 * This code has been modified under the terms listed below and is made
 * available under the same terms.
 */

/*
 *	Copyright 1993-2004 George A Howlett.
 *
 *	Permission is hereby granted, free of charge, to any person obtaining
 *	a copy of this software and associated documentation files (the
 *	"Software"), to deal in the Software without restriction, including
 *	without limitation the rights to use, copy, modify, merge, publish,
 *	distribute, sublicense, and/or sell copies of the Software, and to
 *	permit persons to whom the Software is furnished to do so, subject to
 *	the following conditions:
 *
 *	The above copyright notice and this permission notice shall be
 *	included in all copies or substantial portions of the Software.
 *
 *	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 *	EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *	MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 *	NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 *	LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 *	OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 *	WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef __BltGrAxisOp_h__
#define __BltGrAxisOp_h__

#include "tkbltGraph.h"

namespace Blt {
  extern const Ensemble axisEnsemble[];
  extern int AxisObjConfigure(Blt::Axis* axisPtr, Tcl_Interp* interp,
			      int objc, Tcl_Obj* const objv[]);
};

extern int AxisCgetOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
		      int objc, Tcl_Obj* const objv[]);
extern int AxisConfigureOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			   int objc, Tcl_Obj* const objv[]);
extern int AxisActivateOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			  int objc, Tcl_Obj* const objv[]);
extern int AxisInvTransformOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			      int objc, Tcl_Obj* const objv[]);
extern int AxisLimitsOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			int objc, Tcl_Obj* const objv[]);
extern int AxisMarginOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			int objc, Tcl_Obj* const objv[]);
extern int AxisTransformOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
			   int objc, Tcl_Obj* const objv[]);
extern int AxisTypeOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
		      int objc, Tcl_Obj* const objv[]);
extern int AxisViewOp(Blt::Axis* axisPtr, Tcl_Interp* interp, 
		      int objc, Tcl_Obj* const objv[]);

#endif
